// This file is part of Awali.
// Copyright 2016-2019 Sylvain Lombardy, Victor Marsault, Jacques Sakarovitch
//
// Awali is a free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.

#include <iostream>
#include <unordered_map>

#include <awali/dyn/modules/derivation.hh>

namespace awali { namespace dyn {
/*
 TODO
  std::map<ratexp_t, weight_t> derivation(std::map<ratexp_t, weight_t> e, label_t a, bool breaking = false)
  std::map<ratexp_t, weight_t> derivation(std::map<ratexp_t, weight_t> e, std::string w, bool breaking = false)
  */

  std::map<ratexp_t, weight_t> derivation(ratexp_t exp, label_t a, bool breaking) {
    std::string stat_ctx = exp->get_context()->sname();
    typedef std::map<ratexp_t, weight_t> (*bridge_t)(ratexp_t, label_t, bool);
    static std::unordered_map<std::string, bridge_t> bridges;
    auto it = bridges.find(stat_ctx);
    if(it == bridges.end()) {
      auto bridge = (bridge_t) loading::get_handler("derivation_label", "derivation", stat_ctx);
      bridges.emplace(stat_ctx, bridge);
      return bridge(exp, a, breaking);
    }
    else
      return it->second(exp, a, breaking);
  }

  automaton_t derived_term(ratexp_t exp, bool breaking, bool keep_history) {
    std::string stat_ctx = exp->get_context()->sname();
    typedef automaton_t (*bridge_t)(ratexp_t, bool, bool);
    static std::unordered_map<std::string, bridge_t> bridges;
    auto it = bridges.find(stat_ctx);
    if(it == bridges.end()) {
      auto bridge = (bridge_t) loading::get_handler("derived_term", "derivation", stat_ctx);
      bridges.emplace(stat_ctx, bridge);
      return bridge(exp, breaking, keep_history);
    }
    else
      return it->second(exp, breaking, keep_history);
  }

}}//end of ns awali::dyn
