// This file is part of Awali.
// Copyright 2016-2019 Sylvain Lombardy, Victor Marsault, Jacques Sakarovitch
//
// Awali is a free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.

#include <awali/dyn/modules/graph.hh>

#include <awali/dyn/loading/handler.hh>

namespace awali { namespace dyn {

  std::pair< std::unordered_map<state_t, unsigned int>,
            std::vector<std::vector<state_t>> >
  scc_recursive (automaton_t aut)
  {
    return loading::call<
          std::pair< std::unordered_map<state_t, unsigned int>,
                    std::vector<std::vector<state_t>> >,
          &scc_recursive  >
      ("scc_recursive", "graph", aut);
  }


  std::pair< std::unordered_map<state_t, unsigned int>,
  std::vector<std::vector<state_t>> >
  scc_iterative (automaton_t aut)
  {
    return loading::call<
            std::pair< std::unordered_map<state_t, unsigned int>,
                      std::vector<std::vector<state_t>> >,
            &scc_iterative  >
        ("scc_iterative", "graph", aut);
  }


  std::pair<  automaton_t,
              std::pair< std::unordered_map<state_t, unsigned int>,
                         std::vector<std::vector<state_t>> >  >
  condensation (automaton_t aut)
  {
    return loading::call<
          std::pair< automaton_t,
                     std::pair< std::unordered_map<state_t, unsigned int>,
                                std::vector<std::vector<state_t>> > >,
    &condensation  >
    ("condensation", "graph", aut);
  }

  std::vector<state_t>
  scc_of (automaton_t aut, state_t stt)
  {
    return loading::call2< std::vector<state_t>, state_t, &scc_of >
          ("scc_of", "graph", aut, stt);
  }

}}//end of ns awali::dyn
